#!/usr/bin/env nextflow

// Define input variables
params.rda = "${baseDir}/../test_data/sc10x.rda"
params.seurat = "sc10x"
params.tsne = true
params.umap = true
params.assay = "RNA"
params.slot = "scale.data"
params.scale = true
params.sample = 100
params.groups = "${baseDir}/../test_data/groups.txt"
params.convert = "GRCh38p5"

params.outDir = "${baseDir}/../output"

// Parse input variables
rda = Channel.fromPath(params.rda)
rda.into {
    rda_createMatrix
    rda_createGroup
}
seurat_createMatrix = params.seurat
seurat_createGroup = params.seurat
tsne_createMatrix = params.tsne
tsne_plotIdentGroup = params.tsne
tsne_plotFeature = params.tsne
umap_createMatrix = params.umap
umap_plotIdentGroup = params.umap
umap_plotFeature = params.umap
assay = params.assay
slot = params.slot
scale = params.scale
sample = params.sample
groups_createGroup = file(params.groups).readLines()
groups_plotIdentGroup = file(params.groups).readLines()
groups_plotViolinPlot = file(params.groups)
convert = params.convert

outDir = params.outDir
logsDir = "${outDir}/Logs"

/*
* createMatrix: extract cell/gene matrix from Seurat object
*/
process createMatrix {
    tag "${seurat_createMatrix}"

    input:
    file rda_createMatrix
    val seurat_createMatrix
    val tsne_createMatrix
    val umap_createMatrix
    val assay
    val slot
    val scale
    val sample

    output:
    file("matrix/*") into matrixRaw
    file("cells.csv") into cells

    script:
    """
    hostname
    ulimit -a
    module load seurat/3.0.0
    seurat-Rscript ${baseDir}/scripts/createMatrix.R --r ${rda_createMatrix} --o ${seurat_createMatrix} --t ${tsne_createMatrix} --u ${umap_createMatrix} --a ${assay} --s ${slot} --l ${scale} --p ${sample}
    """
}

/*
* createGroup: extract ident groups from Seurat object
*/
process createGroup {
    tag "${group_createGroup}"
    publishDir "${outDir}/matrix", mode: 'copy', overwrite: true

    input:
    file rda_createGroup
    val seurat_createGroup
    each group_createGroup from groups_createGroup
    file cells
    
    output:
    file("*") into id

    script:
    """
    hostname
    ulimit -a
    module load seurat/3.0.0
    seurat-Rscript ${baseDir}/scripts/createGroup.R --r ${rda_createGroup} --o ${seurat_createGroup} --g ${group_createGroup}
    """
}

//Duplicate group IDs
id.into {
    id_plotIdentGroup
    id_plotViolinBox
}

/*
* renameGenes: convert gene symbols to NCBI ID's for GUDMAP/RBK
*/
process renameGenes {
    tag "NCBI"
    publishDir "${outDir}/matrix", mode: 'copy', overwrite: false

    input:
    file matrixRaw
    val convert

    output:
    file("*") into matrixFinal 
    file("genes.csv") into genes


    script:
    if (convert == "NO"){
        """
        hostname
        ulimit -a
        cp -R ./raw/*.mtx .
        cp -R ./raw/*.csv .
        """
    } else {
        """
        hostname
        ulimit -a
        cp -R ./raw/*.csv .
        module load seurat/3.0.0
        seurat-Rscript ${baseDir}/scripts/renameGenes.R --g ${convert}
        """
    }
}

// Duplicate matrix files
matrixFinal.into {
    matrix_plotIdentGroup
    matrix_plotFeature
    matrix_plotViolinBox
}

/*
* plotIdentGroup: plot identity groups in reduced dimensions
*/
process plotIdentGroup {
    tag "${group_plotIdentGroup}"
    publishDir "${outDir}/png/IdentGroup", mode: 'move', overwrite: true

    input:
    file matrix_plotIdentGroup
    file("*") from id_plotIdentGroup.collect()
    each group_plotIdentGroup from groups_plotIdentGroup
    val tsne_plotIdentGroup
    val umap_plotIdentGroup

    output:
    file("**/*.png") into plotsIdentGroup

    script:
    """
    hostname
    ulimit -a
    module load seurat/3.0.0
    seurat-Rscript ${baseDir}/scripts/plotIdentGroup.R --g ${group_plotIdentGroup} --t ${tsne_plotIdentGroup} --u ${umap_plotIdentGroup}
    """
}

// Split genes into chunks
genes.splitText(by: 1000).into {
    genesChunks_plotFeature
    genesChunks_plotViolinBox
}

/*
* plotFeature: plot feature plots in gene chunks
*/
process plotFeature {
    publishDir "${outDir}/png/Feature", mode: 'move', overwrite: true

    input:
    file matrix_plotFeature
    each file(chunk) from genesChunks_plotFeature
    val tsne_plotFeature
    val umap_plotFeature

    output:
    file("**/*.png") into plotsFeature

    script:
    """
    hostname
    ulimit -a
    module load seurat/3.0.0
    seurat-Rscript ${baseDir}/scripts/plotFeature.R --t ${tsne_plotFeature} --u ${umap_plotFeature} --c ${chunk}
    """
}

/*
* plotViolinBox: plot violin/box plots in gene chunks
*/
process plotViolinBox {
    publishDir "${outDir}/png/ViolinBox", mode: 'move', overwrite: true

    input:
    file matrix_plotViolinBox
    each file(chunk) from genesChunks_plotViolinBox
    file groups_plotViolinPlot
    file("*") from id_plotViolinBox.collect()

    output:
    file("**/*.png") into plotsViolinBox

    script:
    """
    hostname
    ulimit -a
    module load seurat/3.0.0
    while IFS="," read group
    do
        seurat-Rscript ${baseDir}/scripts/plotViolinBox.R --g \${group} --c ${chunk} &
    done < groups.txt
    wait
    """
}
